import pandas as pd
import numpy as np
from letterboxd_stats.cli import render_table, select_sort, select_range


def read_watched_films(df: pd.DataFrame, path: str, name: str):
    df_profile = pd.read_csv(path)
    df.insert(0, "watched", np.where(df["title"].isin(df_profile["Name"]), "[X]", "[ ]"))
    render_table(df, name)


def show_wishlist(path: str, shuffle: bool, limit=None):
    df = pd.read_csv(path)
    df["Year"] = df["Year"].fillna(0).astype(int)
    if shuffle:
        df = df.sample(frac=1)
    if limit is not None:
        df = df.iloc[:limit, :]
    render_table(df, "Wishlist")
    return df


def show_diary(path: str, limit=None):
    df = pd.read_csv(path)
    df["Year"] = df["Year"].fillna(0).astype(int)
    df["Watched Date"] = pd.to_datetime(df["Watched Date"])
    sort_column = select_sort(df.columns.values.tolist())
    df.sort_values(by=sort_column, ascending=False, inplace=True)
    df = df.drop("Rewatch", axis=1)
    df = df.drop("Tags", axis=1)
    if limit is not None:
        df = df.iloc[:limit, :]
    render_table(df, "Diary")
    return df


def show_ratings(path: str, limit=None):
    df = pd.read_csv(path)
    df["Year"] = df["Year"].fillna(0).astype(int)
    df["Date"] = pd.to_datetime(df["Date"])
    sort_column = select_sort(df.columns.values.tolist())
    df.sort_values(by=sort_column, ascending=False, inplace=True)
    if sort_column == "Rating":
        options = df["Rating"].unique().tolist()
        rating_range = select_range(options=options)
        df = df[df["Rating"].isin(rating_range)]
    if limit is not None:
        df = df.iloc[:limit, :]
    render_table(df, "Ratings")
    return df



from rich.console import Console
from rich.table import Table
from rich import box
import pandas as pd
from InquirerPy import inquirer
from InquirerPy.base.control import Choice
from letterboxd_stats import config
from ascii_magic import AsciiArt

IMAGE_URL = "https://www.themoviedb.org/t/p/w600_and_h900_bestv2"


def select_department(departments: list[str], name: str, known_for_department: str) -> str:
    department = inquirer.select(  # type: ignore
        message=f"Select a department for {name}",
        choices=departments,
        default=known_for_department,
    ).execute()
    return department


def select_movie_id(movies_info: pd.DataFrame) -> int:
    movie_id = inquirer.fuzzy(  # type: ignore
        message="Write movie id for more information",
        mandatory=False,
        max_height="25%",
        choices=[
            Choice(value=id, name=f"{id} - {title}") for id, title in zip(movies_info["id"], movies_info["title"])
        ],
        keybindings={"skip": [{"key": "escape"}]},
        validate=lambda result: result in movies_info["id"].values,
        filter=lambda result: None if result is None else int(result),
        invalid_message="Input must be in the resulting IDs",
    ).execute()
    return movie_id


def select_search_result(results: list[str]) -> int:
    choices = [Choice(i, name=r) for i, r in enumerate(results)]
    result = inquirer.select(  # type: ignore
        message="Result of your search. Please select one",
        choices=choices,
        default=choices[0],
    ).execute()
    return result


def select_sort(sort_options: list[str]) -> str:
    result = inquirer.select(  # type: ignore
        message="Select the order of your diary entry:", choices=sort_options, default=sort_options[0]
    ).execute()
    return result


def select_range(options: list[str]) -> list[str]:
    result = inquirer.rawlist(  # type: ignore
        message="Pick a desired value (or select 'all'). Use space to toggle your choices. CTRL+R to select all.",
        choices=options,
        default=options[0],
        multiselect=True,
        validate=lambda result: len(result) > 0,
    ).execute()
    return result


def select_movie(movie_df: pd.DataFrame) -> str:
    result = inquirer.fuzzy(  # type: ingore
        message="Write movie id for more information",
        mandatory=False,
        max_height="25%",
        choices=[
            Choice(value=url, name=f"{title}") for url, title in zip(movie_df["url"], movie_df["title"])
        ],
        keybindings={"skip": [{"key": "escape"}]},
        invalid_message="Input must be in the resulting IDs",
    ).execute()
    return result


def print_film(film):
    grid = Table.grid(expand=True, padding=1)
    grid.add_column()
    grid.add_column()
    for k, v in film.items():
        grid.add_row(str(k), str(v))
    console = Console()
    console.print(grid)


def render_table(df: pd.DataFrame, name: str):
    df_str = df.astype(str)
    table = Table(title=name, box=box.SIMPLE)
    for col in df_str.columns:
        table.add_column(col)
    for _, row in df_str.iterrows():
        table.add_row(*row)
    console = Console()
    console.print(table)


def download_poster(poster: str):
    if config['poster_columns'] > 0:
        art = AsciiArt.from_url(IMAGE_URL + poster)
        art.to_terminal(columns=180)



import os
from zipfile import ZipFile
from letterboxd_stats import config
import requests
from lxml import html

URL = "https://letterboxd.com"
LOGIN_PAGE = URL + "/user/login.do"
DATA_PAGE = URL + "/data/export"


class Downloader:
    def __init__(self):
        self.session = requests.Session()
        self.session.get(URL)

    def login(self):
        request_payload = {
            "username": config["Letterboxd"]["username"],
            "password": config["Letterboxd"]["password"],
            "__csrf": self.session.cookies.get("com.xk72.webparts.csrf"),
        }
        res = self.session.post(LOGIN_PAGE, data=request_payload)
        if res.json()["result"] != "success":
            raise ConnectionError("Impossible to login")

    def download_stats(self):
        res = self.session.get(DATA_PAGE)
        if res.status_code != 200 or "application/zip" not in res.headers["Content-Type"]:
            raise ConnectionError(f"Impossible to download data. Response headers:\n{res.headers}")
        print("Data download successful.")
        filename = res.headers["content-disposition"].split()[-1].split("=")[-1]
        path = os.path.expanduser(os.path.join(config["root_folder"], "static"))
        if not os.path.exists(path):
            os.makedirs(path)
        archive = os.path.join(path, filename)
        with open(archive, "wb") as f:
            f.write(res.content)
        with ZipFile(archive, "r") as zip:
            zip.extractall(path)
        os.remove(archive)


def get_tmdb_id(link: str, is_diary: bool):
    res = requests.get(link)
    movie_page = html.fromstring(res.text)
    if is_diary:
        title_link = movie_page.xpath("//span[@class='film-title-wrapper']/a")
        if len(title_link) > 0:
            movie_link = title_link[0]
            movie_url = URL + movie_link.get("href")
            movie_page = html.fromstring(requests.get(movie_url).text, "lxml")
    tmdb_link = movie_page.xpath("//a[@data-track-action='TMDb']")
    if len(tmdb_link) > 0:
        id = tmdb_link[0].get("href").split("/")[-2]
        return int(id)
    return None



from letterboxd_stats.tmdb import get_person, create_person_dataframe, get_movie_detail
from letterboxd_stats import data
from letterboxd_stats.cli import select_movie_id, select_movie
from letterboxd_stats.web_scraper import Downloader, get_tmdb_id
import os
from letterboxd_stats import args, config


def get_movie_detail_from_url(df, is_diary=False):
    df.rename(columns={"Name": "title", "Letterboxd URI": "url"}, inplace=True)
    link = select_movie(df[["title", "url"]])
    id = get_tmdb_id(link, is_diary)
    if id is not None:
        get_movie_detail(id)


def search_film(args_search: str):
    search_result = get_person(args_search)
    name = search_result["name"]
    try:
        df = create_person_dataframe(search_result)
    except ValueError as e:
        print(e)
        return
    path = os.path.join(config["root_folder"], "static", "watched.csv")
    data.read_watched_films(df, path, name)
    movie_id = select_movie_id(df[["id", "title"]])
    if movie_id is not None:
        get_movie_detail(movie_id)


def get_wishlist(args_random, args_limit):
    path = os.path.join(config["root_folder"], "static", "watchlist.csv")
    df = data.show_wishlist(path, args_random, args_limit)
    get_movie_detail_from_url(df)


def get_diary(args_limit):
    path = os.path.join(config["root_folder"], "static", "diary.csv")
    df = data.show_diary(path, args_limit)
    get_movie_detail_from_url(df, True)


def get_ratings(args_limit):
    path = os.path.join(config["root_folder"], "static", "ratings.csv")
    df = data.show_ratings(path, args_limit)
    get_movie_detail_from_url(df)


def main():
    if args.download:
        ws = Downloader()
        ws.login()
        ws.download_stats()
    if args.search:
        search_film(args.search)
    if args.wishlist:
        get_wishlist(args.random, args.limit)
    if args.diary:
        get_diary(args.limit)
    if args.ratings:
        get_ratings(args.limit)


if __name__ == "__main__":
    main()



from tmdbv3api import TMDb, Person, Movie
import pandas as pd
from letterboxd_stats.cli import select_department, select_search_result, print_film, download_poster
from letterboxd_stats import config

tmdb = TMDb()
tmdb.api_key = config["TMDB"]["api_key"]
person = Person()
movie = Movie()


def get_person(name: str):
    print(f"Searching for '{name}'")
    search_results = person.search(name)
    names = [result.name for result in search_results]  # type: ignore
    if len(names) == 0:
        raise Exception("No results for your search")
    result_index = select_search_result(names)  # type: ignore
    return search_results[result_index]


def create_person_dataframe(search_result) -> pd.DataFrame:
    p = person.details(search_result["id"])
    known_for_department = p["known_for_department"]
    movie_credits = person.movie_credits(search_result["id"])
    list_of_films = [
        {"title": m.title, "release_date": m.release_date, "department": m.department, "id": m.id}
        for m in movie_credits["crew"]
    ]
    if len(list_of_films) == 0:
        raise ValueError("The selected person doesn't have any film.")
    df = pd.DataFrame(list_of_films)
    department = select_department(df["department"].unique(), p["name"], known_for_department)
    df = df[df["department"] == department]
    df = df.drop("department", axis=1)
    df["release_date"] = pd.to_datetime(df["release_date"])
    df.sort_values(by="release_date", inplace=True)
    return df


def get_movie_detail(movie_id: int):
    movie_details = movie.details(movie_id)
    poster = movie_details.get("poster_path")
    if poster is not None:
        download_poster(poster)
    selected_details = {
        "title": movie_details["title"],
        "original_title": movie_details["original_title"],
        "runtime": movie_details["runtime"],
        "overview": movie_details["overview"],
        "release_date": movie_details["release_date"],
    }
    print_film(selected_details)
